import type { AdMainConfigList } from '@/types/Tool'

/**
 * 创建广告
 * config 参数说明
 * @param {string[]} ad_id 广告id
 * @param {string} ad_type 广告类型 1:视频 2:图片
 * @param {string} ad_pos 广告位置
 * @param {string[]} ad_size 广告尺寸
 * @param {string} ad_text 广告文案
 * @param {string[]} ad_url 广告链接
 * @param {string[]} ad_cover 广告图片
 */
export const ad_main = function (config: AdMainConfigList) {
  // 广告sdk
  if (localStorage.getItem('ad_id')) {
    ad_init(config)
    return
  }

  // 生成并输出浏览器指纹
  generateBrowserFingerprint().then((ad_id) => {
    localStorage.setItem('ad_id', ad_id + '')
    ad_init(config)
  })
}

function ad_init(config: AdMainConfigList) {
  for (let i = 0; i < config.length; i++) {
    const v = config[i]
    // 获取对应广告位置
    const ad_pos = document.querySelector('#' + v.ad_pos)
    if (!ad_pos) return console.error('广告位置不存在：' + v.ad_pos)
    const ele = document.createElement('div')
    ele.style.width = v.ad_size[0] + 'px'
    ele.style.height = v.ad_size[1] + 'px'
    // 添加对应广告类型
    switch (v.ad_type) {
      case 1:
        // 广告视频
        ele.innerHTML += `<video src="${v.ad_url}" style="width:100%;height:100%;" />`
        break
      case 2:
        // 广告图片
        ele.innerHTML += `<img src="${v.ad_cover}" style="width:100%;height:100%;" />`
        break
    }
    // 添加广告文案
    const ad_text = document.createElement('div')
    ad_text.innerText = v.ad_text
    ad_text.style.position = 'absolute'
    ad_text.style.bottom = '10px'
    ad_text.style.left = '10px'
    ad_text.style.color = '#fff'
    ad_text.style.fontSize = '14px'
    ele.appendChild(ad_text)
    ele.addEventListener('click', () => {
      // 点击广告
      window.open(v.ad_url)
    })
    ad_pos.appendChild(ele)
  }
}

function generateBrowserFingerprint() {
  return new Promise((resolve, reject) => {
    // 在这里执行获取指纹的操作，例如使用Canvas、WebGL等方法
    // 然后将生成的指纹字符串作为参数传递给resolve函数
    // 获取浏览器信息
    const browserInfo = `${navigator.userAgent} ${window.screen.width}x${window.screen.height}`
    // 获取时区偏移量
    const timezoneOffset = Intl.DateTimeFormat().resolvedOptions().timeZone
    // 在这里处理获取到的指纹信息
    const canvasFingerprint = getCanvasFingerprint()
    const webglFingerprint = getWebglFingerprint()
    // 将所有信息组合成一个字符串
    const fingerprintStr = `${browserInfo}|${timezoneOffset}|${canvasFingerprint}|${webglFingerprint}`
    // 使用createHash生成哈希值
    createHash(fingerprintStr)
      .then((hash) => {
        resolve(hash)
      })
      .catch((error) => {
        reject(error)
      })
  })
}

// Canvas指纹获取函数
function getCanvasFingerprint() {
  const canvas = document.createElement('canvas')
  const ctx = canvas.getContext('2d')
  if (!ctx) return ''
  // 绘制一些内容
  ctx.fillStyle = '#f60'
  ctx.fillRect(0, 0, 16, 16)
  ctx.fillStyle = '#069'
  ctx.font = '11pt Arial'
  ctx.fillText('Cwm fjord!!', 4, 10)
  ctx.fillStyle = 'rgba(102, 204, 0, 0.7)'
  ctx.fillText('Cwm fjord!!', 5, 11)
  ctx.scale(2, 2)
  ctx.fillStyle = '#fff'
  ctx.font = '24pt Arial'
  ctx.fillText('Cwm fjord!!', 10, 25)
  ctx.fillStyle = 'rgba(102, 204, 0, 0.7)'
  ctx.fillText('Cwm fjord!!', 10, 25)

  const data = canvas.toDataURL()
  return data.replace(/data:image\/png;base64,/, '')
}

// WebGL指纹获取函数
function getWebglFingerprint() {
  const canvas = document.createElement('canvas')
  const gl = (canvas.getContext('webgl') ||
    canvas.getContext('experimental-webgl')) as WebGLRenderingContext
  if (!gl) return ''
  const extensions = gl.getSupportedExtensions()
  if (!extensions) return ''
  const extStr = extensions.join(',')
  const rendererInfo = gl.getParameter(gl.RENDERER) + '/' + gl.getParameter(gl.VENDOR)
  return rendererInfo + extStr
}

async function createHash(str: string) {
  // 将字符串转换为 Uint8Array
  const encoder = new TextEncoder()
  const data = encoder.encode(str)

  // 使用 SubtleCrypto API 计算 SHA-256 摘要
  const hashBuffer = await crypto.subtle.digest('SHA-256', data)

  // 将 ArrayBuffer 转换为十六进制字符串
  const hashArray = Array.from(new Uint8Array(hashBuffer))
  const hashHex = hashArray.map((byte) => byte.toString(16).padStart(2, '0')).join('')
  return hashHex
}
